@@ -1,6 +1,8 @@
 package Text::Xslate::Parser;
 use Any::Moose;
+use Any::Moose '::Util::TypeConstraints';
 
+use Scalar::Util ();
 use Text::Xslate::Symbol;
 use Text::Xslate::Util qw(
     $NUMBER $STRING $DEBUG
@@ -54,6 +56,11 @@ my $COMMENT = qr/\# [^\n;]* (?=[;\n])?/xms;
 
 my $CODE    = qr/ (?: (?: $STRING | [^'"] )*? ) /xms; # ' for poor editors
 
+my $RegexpRefType = subtype __PACKAGE__ . '.RegexpRef' => as 'Maybe[RegexpRef]';
+coerce $RegexpRefType =>
+    from 'Str', via { qr/\Q$_\E/xms },
+;
+
 has symbol_table => ( # the global symbol table
     is  => 'ro',
     isa => 'HashRef',
@@ -115,21 +122,24 @@ has input => (
 
 has line_start => (
     is      => 'ro',
-    isa     => 'Maybe[RegexpRef]',
+    isa     => $RegexpRefType,
+    coerce  => 1,
     builder => '_build_line_start',
 );
 sub _build_line_start { qr/\Q:/xms }
 
 has tag_start => (
     is      => 'ro',
-    isa     => 'RegexpRef',
+    isa     => $RegexpRefType,
+    coerce  => 1,
     builder => '_build_tag_start',
 );
 sub _build_tag_start { qr/\Q<:/xms }
 
 has tag_end => (
     is      => 'ro',
-    isa     => 'RegexpRef',
+    isa     => $RegexpRefType,
+    coerce  => 1,
     builder => '_build_tag_end',
 );
 sub _build_tag_end { qr/\Q:>/xms }
@@ -424,6 +434,8 @@ sub init_basic_operators {
 
     $parser->infix('==', 150)->is_logical(1);
     $parser->infix('!=', 150)->is_logical(1);
+    $parser->infix('<=>', 150);
+    $parser->infix('cmp', 150);
 
     $parser->infix('|',  140, \&led_bar);
 
@@ -479,7 +491,7 @@ sub init_symbols {
 
     $parser->symbol('include')  ->set_std(\&std_include);
 
-    # template inheritance
+    # macros
 
     $parser->symbol('cascade')  ->set_std(\&std_cascade);
     $parser->symbol('macro')    ->set_std(\&std_proc);
@@ -490,7 +502,9 @@ sub init_symbols {
     $parser->symbol('super')    ->set_std(\&std_marker);
     $parser->symbol('override') ->set_std(\&std_override);
 
-    # lexical variable stuff
+    $parser->symbol('->')       ->set_nud(\&nud_lambda);
+
+    # lexical variables/constants stuff
     $parser->symbol('constant')->set_nud(\&nud_constant);
     $parser->symbol('my'      )->set_nud(\&nud_constant);
 
@@ -1081,6 +1095,56 @@ sub nud_constant {
     );
 }
 
+# -> $x { ... }
+sub nud_lambda {
+    my($parser, $symbol) = @_;
+
+    my $unique_name = $parser->symbol('(name)')->clone(
+        id => sprintf('lambda@0x%x', Scalar::Util::refaddr($symbol)),
+    );
+
+    my $pointy = $symbol->clone(
+        arity => 'proc',
+        id    => 'macro',
+        first => $unique_name, # name
+    );
+
+    $parser->new_scope();
+    my @params;
+    if($parser->token->id ne "{") { # has params
+        my $paren = ($parser->token->id eq "(");
+
+        $parser->advance("(") if $paren; # optional
+
+        my $t = $parser->token;
+        while($t->arity eq "variable") {
+            push @params, $t;
+            $parser->define($t);
+
+            $t = $parser->advance();
+            if($t->id eq ",") {
+                $t = $parser->advance(); # ","
+            }
+            else {
+                last;
+            }
+        }
+
+        $parser->advance(")") if $paren;
+    }
+    $pointy->second( \@params );
+
+    $parser->advance("{");
+    $pointy->third($parser->statements());
+    $parser->advance("}");
+    $parser->pop_scope();
+
+    return $symbol->clone(
+        arity => 'lambda',
+        first => $pointy,
+    );
+}
+
 #sub std_var {
 #    my($parser, $symbol) = @_;
 #    my @a;
@@ -1115,14 +1179,14 @@ sub nud_constant {
 # ->      { STATEMENTS }
 #         { STATEMENTS }
 sub pointy {
-    my($parser, $node, $in_for) = @_;
+    my($parser, $pointy, $in_for) = @_;
 
-    my @vars;
+    my @params;
 
     $parser->new_scope();
 
     if($parser->token->id eq "->") {
-        $parser->advance("->");
+        $parser->advance();
         if($parser->token->id ne "{") {
             my $paren = ($parser->token->id eq "(");
 
@@ -1130,7 +1194,7 @@ sub pointy {
 
             my $t = $parser->token;
             while($t->arity eq "variable") {
-                push @vars, $t;
+                push @params, $t;
                 $parser->define($t);
 
                 if($in_for) {
@@ -1150,10 +1214,10 @@ sub pointy {
             $parser->advance(")") if $paren;
         }
     }
-    $node->second( \@vars );
+    $pointy->second( \@params );
 
     $parser->advance("{");
-    $node->third($parser->statements());
+    $pointy->third($parser->statements());
     $parser->advance("}");
     $parser->pop_scope();
 
@@ -1564,6 +1628,7 @@ sub _error {
         $parser->file, $parser->line, $message, $near);
 }
 
+no Any::Moose '::Util::TypeConstraints';
 no Any::Moose;
 __PACKAGE__->meta->make_immutable;
 __END__
