@@ -52,10 +52,13 @@ has value => (
 
 # flags
 has [
+        'is_reserved',  # set by reserve()
+        'is_defined',   # set by define()
         'is_block_end', # block ending markers
         'is_logical',   # logical operators
         'is_comma',     # comma like operators
         'is_value',     # symbols with values
+        'is_statement', # expr but a statement (e.g. assignment)
     ] => (
     is       => 'rw',
     isa      => 'Bool',
@@ -141,13 +144,6 @@ has assignment => (
     required => 0,
 );
 
-has reserved => (
-    is  => 'rw',
-    isa => 'Bool',
-
-    required => 0,
-);
-
 #has scope => (
 #    is  => 'rw',
 #    isa => 'HashRef',
